<?php

declare(strict_types=1);

namespace Drupal\nodepermissions\Hook;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Session\AccountInterface;

/**
 * Hook implementations for nodepermissions.
 */
class NodeAccessHooks {

  /**
   * Grants edit access to administrative fields.
   *
   * @see nodepermissions.permissions.yml
   */
  #[Hook('entity_field_access')]
  public function entityFieldAccess($operation, FieldDefinitionInterface $field_definition, AccountInterface $account): AccessResultInterface {
    // Only apply to the edit operation.
    if ($operation !== 'edit') {
      return AccessResult::neutral();
    }

    // Only affect nodes.
    if ($field_definition->getTargetEntityTypeId() !== 'node') {
      return AccessResult::neutral();
    }

    // Only affect these fields.
    // @see Drupal\node\NodeAccessControlHandler::checkFieldAccess
    $administrative_fields = ['uid', 'status', 'created', 'promote', 'sticky'];

    $field_name = $field_definition->getName();
    if (!in_array($field_name, $administrative_fields, TRUE)) {
      return AccessResult::neutral();
    }

    return AccessResult::allowedIfHasPermission($account, "administer node $field_name");
  }

}
